;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                TRAP MANAGER            ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Trap Adress Name
.global __OscillatorFail
.global __AddressError
.global __StackError
.global __MathError
.global __DMACError

.global _guessedBadPCLow
.global _guessedBadPCHigh

.section .bss
_guessedBadPCLow: .space 2
_guessedBadPCHigh: .space 2

.section .text

;-------------------------------------------------------------------------------------
; OSCILLATOR FAILURE
;
;-------------------------------------------------------------------------------------
__OscillatorFail:
StayTrappedOscillatorFail:                     ;Stay in this routine
        bra     StayTrappedOscillatorFail


;-------------------------------------------------------------------------------------
; ADDRESS ERROR
; - Extract the last ProgramCounter in _guessedBadPCLow _guessedBadPCHigh
; - Wait here for debugging
;-------------------------------------------------------------------------------------
__AddressError:
        mov     #_guessedBadPCHigh, w1
        pop     [w1--]                  ;Pop the Program Counter (PC) from the stack
        pop     [w1++]                  ;Remember that the PC(PCH and PCL) is stacked
        bclr    [w1], #7                ;along with the SRL byte and IPL3 bit from
        inc     w1, w1                  ;CORCON. So we need to extract just the 24-bit
        clr.b   [w1]                    ;PCH:PCL information from the stack
        mov     #_guessedBadPCLow, w1  ;Decrement that value by 2
        mov     #2, w2                  ;Store the 24-bit result into 2x16-bit words
        subr    w2, [w1], [w1++]        ;in RAM at- guessedBadPCHigh:Low
        clr     w2
        subbr   w2, [w1], [w1]
        bclr    INTCON1, #3       	;Clear the trap flag
StayTrappedAddrErr:                     ;Stay in this routine
        bra     StayTrappedAddrErr
        ;Place a breakpoint above and halt the Debugging process and view
        ;the variables named: guessedBadPCHigh:Low
        retfie


/*
Copyright (C) 2009 -  Christian Nastasi
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

;-------------------------------------------------------------------------------------
; STACK ERROR
;
;-------------------------------------------------------------------------------------
__StackError:
StayTrappedStackError:                     ;Stay in this routine
        bra     StayTrappedStackError

;-------------------------------------------------------------------------------------
; MATHEMATIC ERROR
;
;-------------------------------------------------------------------------------------
__MathError:
StayTrappedMathError:                     ;Stay in this routine
        bra     StayTrappedMathError

;-------------------------------------------------------------------------------------
; DMA CHANNEL ERROR
;
;-------------------------------------------------------------------------------------
__DMACError:
StayTrappedDMACError:                     ;Stay in this routine
        bra     StayTrappedDMACError


